#!/usr/bin/env python

# internal libraries
import os
from time import sleep
# interface libraries
from Tkinter import *
from ttk import *
import tkFileDialog as fd
# my files
from main import main
from getboundary import getboundary
from list_set import list_set
from sum_binary import sum_binary

def makeform(root, fields):
	entries = {}
	rows= []
	for field in fields:
		row = Frame(root)
		row.pack(side=TOP, fill=X, padx=5, pady=5)
		if  field =='Merge CellProfiler Segmented Objects' or field == 'Build Model' or field =='' or field == 'Apply Model': 
			lab = Label(row, width=30, text=field, anchor='w',font=("Helvetica", 16))
			lab.pack(side=LEFT)
		else:
			ent = Entry(row)
			if field == 'Number of eigen-shapes':
				ent.insert(0,"choose a number")
			elif field == 'Status':
				ent.insert(0,'welcome to the picellcounter')
			elif field == 'Name of the model':
				ent.insert(0,'name your model')
			elif field == 'Image sets for building' or field == 'Image sets for applying' or field == 'List of segmented images':
				ent.insert(0,'<--- click to load csv')
			elif field == 'Model to apply':
				ent.insert(0,'<--- click to load model')
			else:ent.insert(0,"<--- click to search folder")
			ent.pack(side=RIGHT, expand=YES, fill=X)
			entries[field] = ent
			lab = Label(row, width=24, text=field, anchor='w')
			lab.pack(side=LEFT)
		rows.append(row)
	return entries,rows

def getdir(entries,target):
	entries['Status'].delete(0,END)
	entries['Status'].insert(0,'searching...')
	#################################################################
	folder = StringVar()
	foldername = fd.askdirectory()
	folder.set(foldername)
	folder = folder.get()
	entries[target].delete(0,END)
	entries[target].insert(0,folder)
	#################################################################
	entries['Status'].delete(0,END)
	entries['Status'].insert(0,'directory found...')

def getcsv(entries,target):
	entries['Status'].delete(0,END)
	entries['Status'].insert(0,'searching...')
	#################################################################
	folder = StringVar()
	foldername = fd.askopenfilename()
	folder.set(foldername)
	folder = folder.get()
	entries[target].delete(0,END)
	entries[target].insert(0,folder)
	#################################################################
	entries['Status'].delete(0,END)
	entries['Status'].insert(0,'directory found...')

def merge(entries,progress_bar):
	entries['Status'].delete(0,END)
	entries['Status'].insert(0,'merging...')
	#################################################################
	CPsubfolder = entries['Locate CellProfiler objects'].get() 
	CPsubfolder = os.path.abspath(CPsubfolder)
	# 'C:\\Users\\kuki\\Desktop\\cpoutput315\\Experiment1'
	labeledimfolder = os.path.join(CPsubfolder,'segmented images')
	if not os.path.isdir(labeledimfolder):
		os.mkdir(labeledimfolder)
		sum_binary(entries,CPsubfolder,progress_bar)
	#################################################################
	folder = os.path.join(CPsubfolder,'segmented images')
	list_set(folder)
	progress_bar["value"] = 100
	progress_bar.update()
	entries['Status'].delete(0,END)
	entries['Status'].insert(0,'merging completed...')

# def createobjectlist(entries,progress_bar):
# 	entries['Status'].delete(0,END)
# 	entries['Status'].insert(0,'creating object csv...')
# 	#################################################################
# 	# input definition
# 	folder = entries['List of segmented images'].get()
# 	# function
# 	folder = os.path.dirname(folder)
# 	getboundary(folder,progress_bar,entries)
# 	#################################################################
# 	progress_bar["value"] = 100
# 	progress_bar.update()
	

def Model(entries,BuildModel,progress_bar):
	entries['Status'].delete(0,END)
	entries['Status'].insert(0,'modeling initiated...')
	#################################################################
	# input definition
	if BuildModel == True:
		csv = entries['Image sets for building'].get() 
		clnum = entries['Number of eigen-shapes'].get()
		modelname = entries['Name of the model'].get() #name
	else:
		csv = entries['Image sets for applying'].get()
		modelname = entries['Model to apply'].get()  #path
		clnum=None 
	# function
	# folder = os.path.dirname(folder)
	getboundary(csv,progress_bar,entries)
	main(BuildModel,csv,entries,modelname,clnum,progress_bar)
	#################################################################
	progress_bar["value"] = 100
	progress_bar.update()
	entries['Status'].delete(0,END)
	entries['Status'].insert(0,'modeling completed...')

def cellcounter(): 
	root = Tk()
	root.style = Style()
	root.style.theme_use('clam')
	root.configure(background='#dcdad5')
	root.style.configure("red.Horizontal.TProgressbar", troughcolor ='gray',  background='#EA6676')
	root.title("picellcounter")
	fields = ('Merge CellProfiler Segmented Objects','Locate CellProfiler objects','', #merge button
		'Build Model','Image sets for building', 'Number of eigen-shapes','Name of the model','', #build model button
		'Apply Model','Image sets for applying','Model to apply','','Status','')
	ents,rows = makeform(root, fields)
	root.bind('<Return>', (lambda event, e=ents: fetch(e))) 
	#####################################################################################################################
	progress_bar = Progressbar(rows[13], style="red.Horizontal.TProgressbar", orient="horizontal", mode="determinate",maximum=100)
	progress_bar.pack(side=RIGHT,padx=5,pady=5)
	#####################################################################################################################
	#function 1 : merge
	b1 = Button(rows[1],text='search folder', width=12,command=(lambda e=ents: getdir(e,'Locate CellProfiler objects')))
	b1.pack(side=RIGHT,padx=5,pady=5)
	#function 2
	#csvgen = BooleanVar()
	# checkb = Checkbutton(rows[2],text='create list of image sets',variable=csvgen)
	b2 = Button(rows[2],text='merge', width=6,command=(lambda e=ents: merge(e,progress_bar)))
	b2.pack(side=RIGHT,padx=5,pady=5)
	# checkb.pack(side=RIGHT,padx=5,pady=5)
	#####################################################################################################################
	# #function 3 : create csv
	# b3 = Button(rows[4],text='load csv', width=12,command=(lambda e=ents: getcsv(e,'List of segmented images')))
	# b3.pack(side=RIGHT,padx=5,pady=5)
	# #function 4_1 : create image csv
	# b4 = Button(rows[5],text='create',width=6,command=(lambda e=ents: createobjectlist(e,progress_bar)))
	# b4.pack(side=RIGHT,padx=5,pady=5)
	#####################################################################################################################
	#function 5 : Model 
	b5 = Button(rows[4],text='load csv', width=12,command=(lambda e=ents: getcsv(e,'Image sets for building')))
	b5.pack(side=RIGHT,padx=5,pady=5)
	#function 6
	b6 = Button(rows[7],text='build model',width=12,command=(lambda e=ents: Model(e,True,progress_bar)))
	b6.pack(side=RIGHT,padx=5,pady=5)
	#function 7
	b7 = Button(rows[9],text='load csv', width=12,command=(lambda e=ents: getcsv(e,'Image sets for applying')))
	b7.pack(side=RIGHT,padx=5,pady=5)
	#function 8
	b8 = Button(rows[10],text='load model', width=12,command=(lambda e=ents: getdir(e,'Model to apply')))
	b8.pack(side=RIGHT,padx=5,pady=5)
	#function 9
	b9 = Button(rows[11],text='apply model',width=12,command=(lambda e=ents: Model(e,False,progress_bar)))
	b9.pack(side=RIGHT,padx=5,pady=5)
	#####################################################################################################################
	#terminate
	quit = Button(root, text='Quit',width=6, command=root.quit)
	quit.pack(side=LEFT, padx=5, pady=5)
	root.mainloop()



import io
import socket
import struct
from PIL import Image

# Start a socket listening for connections on 0.0.0.0:8000 (0.0.0.0 means
# all interfaces)
server_socket = socket.socket()
server_socket.bind(('0.0.0.0', 8000))
server_socket.listen(0)

# Accept a single connection and make a file-like object out of it
connection = server_socket.accept()[0].makefile('rb')
try:
    while True:
        # Read the length of the image as a 32-bit unsigned int. If the
        # length is zero, quit the loop
        image_len = struct.unpack('<L', connection.read(struct.calcsize('<L')))[0]
        if not image_len:
            break
        # Construct a stream to hold the image data and read the image
        # data from the connection
        image_stream = io.BytesIO()
        image_stream.write(connection.read(image_len))
        # Rewind the stream, open it as an image with PIL and do some
        # processing on it
        image_stream.seek(0)
        image = Image.open(image_stream)
        print('Image is %dx%d' % image.size)
        image.verify()
        print('Image is verified')
finally:
    connection.close()
    server_socket.close()


#!/usr/bin/env python

# internal libraries
from __future__ import division
from copy import deepcopy
from inspect import getargspec
import time
import os
# external libraries
import pandas as pd
import numpy as np
from scipy import stats, cluster, spatial
from sklearn.cluster import KMeans
from sklearn import preprocessing
import matplotlib as mpl
mpl.use("TkAgg")
import matplotlib.pyplot as plt
from matplotlib.ticker import NullFormatter
# my files
from PCA import *

def clusterSM(folder,modelname,score,pc,bdpc,clnum=None,pcnum=None,VamModel=None,BuildModel=None,ch=None,condition=None):
    print('# clusterSM')
    np.set_printoptions(precision=4,suppress=True)
    #desktop = os.path.join(os.environ["HOMEPATH"],"Desktop")
    folder = os.path.abspath(folder)
    desktop = os.path.join(folder,modelname)
    if BuildModel : desktop = os.path.join(desktop,ch+'_buildmodel')
    else : desktop = os.path.join(desktop,ch+'_applymodel')
    if not os.path.exists(desktop):
        os.makedirs(desktop)
    Nuu=int(round(len(bdpc.T[0])))
    Nbb=int(round(len(bdpc[0]) / 2))
    mmx=np.dot(np.ones([Nuu,1]),np.mean([bdpc],axis=1))
    smx=np.ones(bdpc.shape)
    mdd=mmx[0]
    sdd=smx[0]
    mss=np.mean(score,axis=0)
    sss=np.std(score,axis=0)

    NN=10

    if clnum==None:
        clnum=15
    if pcnum==None:
        pcnum=20

    if BuildModel:
        VamModel['clnum']=clnum
        VamModel['pcnum']=pcnum
    else:
        clnum = VamModel['clnum']
        pcnum = VamModel['pcnum']
        pc = VamModel['pc']

    cms00 = score[:,0:pcnum]
    cms = deepcopy(cms00)

    if BuildModel:
        mincms=np.amin(cms,axis=0)
        VamModel['mincms']=mincms
        VamModel['boxcoxlambda']=np.zeros(len(cms.T))
        VamModel['testmean']=np.zeros(len(cms.T))
        VamModel['teststd']=np.zeros(len(cms.T))
    else:
        mincms=VamModel['mincms']

    for k in range(len(cms.T)):
        test=cms.T[k]
        test=test - mincms[k] + 1
        if BuildModel:
            test,maxlog=stats.boxcox(test)
            test = np.asarray(test)
            #####################
            VamModel['boxcoxlambda'][k]=maxlog
            VamModel['testmean'][k]=np.mean(test)
            VamModel['teststd'][k]=np.std(test)
            #####################
            cms.T[k]=(test-np.mean(test))/np.std(test)
        else:
            test=stats.boxcox(test,VamModel['boxcoxlambda'][k])
            cms.T[k]=(test-VamModel['testmean'][k])/VamModel['teststd'][k]

    cms0=deepcopy(cms)
    cmsn=deepcopy(cms)

    if BuildModel:
        cmsn_Norm = preprocessing.normalize(cmsn)
        if isinstance(clnum, basestring):
            clnum = int(clnum)

        kmeans=KMeans(n_clusters=clnum,init='k-means++',n_init=3,max_iter=300).fit(cmsn_Norm) #init is plus,but orginally cluster, not available in sklearn
        IDX = kmeans.labels_
        C = kmeans.cluster_centers_
        ##################
        VamModel['C']=C
        D = spatial.distance.cdist(cmsn,VamModel['C'])
        IDX = np.argmin(D,axis=1)
    else:
        C = VamModel['C']
        D = spatial.distance.cdist(cmsn,VamModel['C'])
        IDX = np.argmin(D,axis=1)
        ##################
    clnum=max(IDX)+1
    offx,offy=np.meshgrid(range(clnum),[0])
    offx=np.multiply(offx,1)+1
    offx=offx[0]*1-0.5
    offy=np.subtract(np.multiply(offy,1), 1.5)+1
    offy=offy[0]
    cmap = plt.cm.jet
    vmax = int(clnum*10)
    norm = mpl.colors.Normalize(vmin=0,vmax=vmax)
    cid = plt.cm.ScalarMappable(norm=norm,cmap=cmap)

    clshape=np.zeros([clnum,len(cms00[0])])
    clshapesdv=deepcopy(clshape)
    bdst0=np.empty(len(bdpc.T))
    bdst=deepcopy(bdst0)
    for kss in range(clnum):
        c88=IDX == kss
        bdpcs=bdpc[c88,:]
        mbd=np.mean(bdpcs,axis=0)
        bdst0=np.vstack((bdst0,mbd))
    bdst0=bdst0[1:]
    # dendrogram of the difference between different shape
    if not BuildModel:
        Z = VamModel['Z']

    else:
        Y=spatial.distance.pdist(bdst0,'euclidean')
        Z=cluster.hierarchy.linkage(Y,method='complete') #4th row is not in matlab
        Z[:,2] = Z[:,2]*5 #multiply distance manually 10times to plot better.
        VamModel['Z'] = Z

    cluster.hierarchy.set_link_color_palette(['k'])
    fig = plt.figure(289,figsize=(6,2),linewidth=2.0,frameon=False)
    plt.yticks([])
    R=cluster.hierarchy.dendrogram(Z,p=0,truncate_mode='mlab',orientation='bottom',ax=None,above_threshold_color='k')
    leaflabel = np.array(R['ivl'])
    dendidx = leaflabel
    cluster.hierarchy.set_link_color_palette(None)
    if BuildModel : fig.savefig(os.path.join(desktop,"eigen-shape dendrogram with ID number_"+ch+".png"), dpi=1200)
        

    plt.axis('equal')
    plt.axis('off')
    IDXsort=np.zeros(len(IDX))
    for kss in range(clnum):
        c88=IDX == int(dendidx[kss])
        IDXsort[c88]=kss
    IDX=deepcopy(IDXsort)
    bdsubtype=np.empty((int(max(IDX)+1),2,Nbb+1)) #need more specific preallocation: 2 for x and y, Nbb+1 for len(x)
    for kss in range(int(max(IDX))+1):
        c88=IDXsort == kss
        clshape[kss]=np.mean(cms00[c88],axis=0)
        clshapesdv[kss]=np.std(cms00[c88],axis=0)
        pnn=np.zeros(len(pc.T[0]))
        for kev in range(len(cms00[0])):
            pnn=np.add(pnn,np.multiply(pc.T[kev],clshape[kss,kev]))
            pnnlb=np.add(pnn,np.multiply(np.multiply(pc.T[kev],-2),clshapesdv[kss,kev]))
            pnnhb=np.add(pnn,np.multiply(np.multiply(pc.T[kev],2),clshapesdv[kss,kev]))
        pnn=np.multiply(pnn,sdd) + mdd
        pnnlb=np.multiply(pnnlb,sdd) + mdd
        pnnhb=np.multiply(pnnhb,sdd) + mdd  #pnn,pnnlb&hb are all randomized
        xx=pnn[0:Nbb]
        yy=pnn[Nbb:]
        xlb=pnnlb[0:Nbb]
        ylb=pnnlb[Nbb:]
        xhb=pnnhb[0:Nbb]
        yhb=pnnhb[Nbb:]
        xx=np.append(xx,xx[0])
        yy=np.append(yy,yy[0])
        xxhb=np.append(xhb,xhb[0])
        yyhb=np.append(yhb,yhb[0])
        xxlb=np.append(xlb,xlb[0])
        yylb=np.append(ylb,ylb[0])
        fss=4
        plt.figure(289,figsize=(6,3))
        plt.axis('equal')
        plt.plot((xx/fss+offx.T[kss])*10, (yy/fss+offy.T[kss])*10, '-',color=cid.to_rgba(kss)) #this is not plotted in matlab as well
        bdsubtype[kss][0]=xx/fss
        bdsubtype[kss][1]=yy/fss
        bdpcs=bdpc[c88]
        mbd=np.mean(bdpcs,axis=0)
        bdNUM=int(round(len(mbd)/2))
        bdst=np.vstack((bdst,mbd))
        xaxis = np.add(np.divide(np.append(mbd[0:bdNUM],mbd[0]),fss),offx[kss])*10
        yaxis = np.add(np.divide(np.append(mbd[bdNUM:],mbd[bdNUM]),fss),offy[kss])*10
        figeach= plt.figure(7)
        plt.plot(xaxis,yaxis,'-')
        plt.gca().invert_yaxis()
        plt.axis('off')
        if BuildModel : figeach.savefig(os.path.join(desktop,"eigen-shape "+dendidx[kss]+".png"), dpi=1200)
        plt.clf()
        plt.figure(289,figsize=(6,3))
        plt.plot(xaxis,yaxis,'-') #this is the shape of the dendrogram
        sid=np.argsort(np.random.rand(sum(c88),1),axis=0)
        if len(sid)<NN: enum = len(sid)
        else:enum = NN
        for knn in range(enum):
            plt.figure(922)
            x99=bdpcs[sid[knn],np.append(range(bdNUM),0)]
            y99=bdpcs[sid[knn],np.append(np.arange(bdNUM,(bdNUM*2),1),bdNUM)]
            xax = np.add(np.divide(x99,fss),offx[kss])
            yax = np.add(np.divide(y99,fss),offy[kss])
            plt.plot(xax,yax,'r-',linewidth=0.1)
    bdst = bdst[1:]
    # result visulation
    fig922=plt.figure(922, figsize=(15,3))
    plt.axis('equal')
    plt.axis('off')
    if BuildModel : fig922.savefig(os.path.join(desktop,"eigen-shapes_"+ch+".png"), dpi=1200)
    fig289=plt.figure(289,figsize=(6,3))
    plt.axis('equal')
    plt.axis('off')
    if BuildModel : fig289.savefig(os.path.join(desktop,"eigen-shape dendrogram_"+ch+".png"), dpi=1200)
    crap=np.sort(IDX)
    sid=np.argsort(IDX)
    BMsort=cmsn[sid]
    fig20=plt.figure(20)
    plt.imshow(cmsn,extent=[0,1,0,1])
    plt.clim(-1,1)
    plt.gca().xaxis.set_minor_formatter(NullFormatter())
    plt.yticks()
    plt.gca().yaxis.set_minor_formatter(NullFormatter())
    fig21=plt.figure(21)
    plt.imshow(BMsort,extent=[0,1,0,1])
    plt.gca().tick_params(direction='out')
    plt.yticks()
    plt.gca().yaxis.set_minor_formatter(NullFormatter())
    fig23=plt.figure(23)
    crap = np.expand_dims(crap,axis=1)
    imgplot = plt.imshow(crap,cmap='jet',extent=[0,1,0,1]) #imagesc(crap)
    plt.axis('on')
    plt.xticks()
    plt.gca().xaxis.set_minor_formatter(NullFormatter())
    plt.yticks()
    plt.gca().yaxis.set_minor_formatter(NullFormatter())
    plt.close()
    n,bins,patches = plt.hist(IDX,range(clnum+1))

    # sortedheights =  [n[i] for i in np.array(R['leaves'])]
    # sortedbins = np.array(R['ivl'])

    # plt.close()
    # fig22 = plt.figure(22, figsize = (10,5))
    # sortedheights = np.divide(sortedheights,np.sum(sortedheights))
    # sortedheights = np.multiply(sortedheights,100)
    # sortedheights = np.around(sortedheights,2)
    # print clnum
    # print sortedbins
    # plt.bar(x=sortedbins,height=sortedheights,width=0.5,align='center',color='blue',edgecolor='black')
    # plt.xticks(np.array(R['leaves']))
    # plt.ylabel('%')
    # fig22.savefig(os.path.join(desktop,'eigen-shape distribution_'+ch+'.png'))
    # plt.close('all')
    plt.close()
    fig22 = plt.figure(22,figsize = (10,5))
    n=np.divide(n,np.sum(n))
    n=np.multiply(n,100)
    n=np.around(n,2)
    plt.bar(x=np.delete(bins,0)-1,height=n,width=0.5,align='center',color='lightblue',edgecolor='black')

    #plt.xticks(np.delete(bins,0)-1)
    plt.xticks([])
    plt.yticks([])
    #plt.ylabel('%')

    # only for paper
    plt.ylim((0,50)) 
    # only for paper
    ax = fig22.add_subplot(111)

    for i,v in enumerate(n):
        ax.text(i-.25,v+3, str(v),color='blue',fontweight='bold')

    for axis in ['top','bottom','left','right']:
        ax.spines[axis].set_linewidth(2)

    if BuildModel:
        fig22.savefig(os.path.join(desktop,'eigen-shape distribution_'+ch+'_model_'+'.png'), dpi=1200)
    else:
        fig22.savefig(os.path.join(desktop,'eigen-shape distribution_'+ch+'_'+condition+'.png'), dpi=1200)
    plt.close('all')

    return IDX,bdsubtype,C,VamModel

def cluster_main(folder,modelname,score,pc,bdpc,clnum=None,pcnum=None,VamModel=None,BuildModel=None,cellornuc=None,condition=None):
    print('## clusterSM.py')
    start = time.time()
    IDX,bdsubtype,C,VamModel=clusterSM(folder,modelname,score,pc,bdpc,clnum,pcnum,VamModel,BuildModel,cellornuc,condition)
    end = time.time()
    print('For cluster, elapsed time is ' + str(end-start) + 'seconds...')
    return IDX,bdsubtype,C,VamModel


#!/usr/bin/env python

# built-in libraries
import os
from time import sleep
import re
from Tkinter import END
# external libraries
from PIL import Image
from scipy.ndimage.measurements import center_of_mass, label
from scipy.ndimage import generate_binary_structure
import numpy as np
import cv2
import pandas as pd

def createimstack(ch,setfolder):
	imlist = [_ for _ in os.listdir(setfolder) if _.lower().endswith('.tiff')]
	imlist = [_ for _ in imlist if re.split('_+',_)[0].lower() == ch.lower()] 	##fixpoint
	#imlist.sort(key=lambda f: int(filter(str.isdigit,f)))
	imlist = sorted(imlist)
	imlistpath = [os.path.join(setfolder,_) for _ in imlist]
	imstack = [np.array(Image.open(im)) for im in imlistpath]
	return imstack,imlistpath,imlist

def check_label_status(im):
	imlabel = list(set(im.flatten()))[1:]
	if len(imlabel) > 1: tag='labeled'
	return tag

def mask2boundary(mask):
	im2, contours, hierarchy = cv2.findContours(mask,cv2.RETR_TREE,cv2.CHAIN_APPROX_SIMPLE)
	contour = np.empty(2)
	for i in range(len(contours[0])):
		contour = np.vstack((contours[0][i][0],contour))
	contour=contour[0:-1]
	contour.T[0] = contour.T[0]+1
	contour.T[1] = contour.T[1]+1
	boundary = np.empty((2,len(contour.T[1])))
	boundary[0] = contour.T[1]
	boundary[1] = contour.T[0]
	boundary = boundary.T.astype(int)
	return boundary

def getboundary(folder,progress_bar,entries):
	sets = [_ for _  in os.listdir(folder) if os.path.isdir(os.path.join(folder,_))]
	setpaths = [os.path.join(folder,_) for _ in sets]
	for setfolderidx, setfolder in enumerate(setpaths):
		progress_bar["value"] = 100*setfolderidx/len(setpaths)
		progress_bar.update()
		UI = pd.read_csv(os.path.join(folder,'segmented image set location.csv'))
		ch1= UI['ch1'][setfolderidx]
		ch2= UI['ch2'][setfolderidx]
		chlist = [ch1,ch2]
		for ch in chlist:
			imstack,imlistpath,imlist = createimstack(ch,setfolder)
			try: 
				inputim = check_label_status(imstack[0]) 
			except:
				entries['Status'].delete(0,END)
				entries['Status'].insert(0,'The channel columns in the segmented image set location.csv need to be updated')
				return
			if inputim is not 'labeled':
				s = generate_binary_structure(2,2)
				imstack = [label(im,structure=s)[0] for im in imstack]
			cmasses=[]
			cmassdst = os.path.join(setfolder,ch+'_centroid.pickle')
			boundarymaster=[]
			boundarydst = os.path.join(setfolder,ch+'_boundary_coordinate_stack.pickle')
			if os.path.exists(boundarydst): continue
			if os.path.exists(cmassdst): continue
			for imidx, im in enumerate(imstack):
				labels = list(set(im.flatten()))[1:]
				for objidx,lab in enumerate(labels):
					mask = np.array((im==lab).astype(int),dtype='uint8')
					if not os.path.exists(cmassdst):
						cmass = [int(np.around(_,0)) for _ in center_of_mass(mask)]
						cmass.reverse()
						area = [int(np.sum(mask))]
						fronttag = [imlist[imidx],imidx+1,objidx+1]
						cmass = fronttag + cmass + area
						cmasses.append(cmass)
					if not os.path.exists(boundarydst):
						boundary = mask2boundary(mask)
						if len(boundary)<10:
							print 'len is too low'
							continue
						boundarymaster.append(boundary)
			if not os.path.exists(boundarydst):
				df = pd.DataFrame(boundarymaster)
				df.to_pickle(boundarydst)
			if not os.path.exists(cmassdst):
				df_cmass = pd.DataFrame(cmasses)
				df_cmass.columns = ['Filename','ImageID','ObjectID','X','Y','Area']
				df_cmass.index = df_cmass.index+1
				df_cmass.to_csv(os.path.join(setfolder,ch+'_registry.csv'))
	entries['Status'].delete(0,END)
	entries['Status'].insert(0,'object csv created...')
	return



#!/usr/bin/env python

# built-in libraries
import os
from time import sleep
# external libraries
import pandas as pd
import numpy as np

def modeling_ledger(exp):
	d= {'condition': [],
		'set number':[],
		'set location':[],
		'note':[],
		'ch1':[],
		'ch2':[]}	
	emptycsv_df = pd.DataFrame(data=d)
	emptycsv_df = emptycsv_df[['condition','set number','set location','note','ch1','ch2']]
	emptycsv_dst1 = os.path.join(exp,'image sets to build model.csv')
	emptycsv_dst2 = os.path.join(exp,'image sets to apply model.csv')
	emptycsv_df.to_csv(emptycsv_dst1, index=False)
	emptycsv_df.to_csv(emptycsv_dst2, index=False)

def list_set(folder): #folder == cpoutput folder
	folder = os.path.abspath(folder)
	setfolders = [_ for _ in os.listdir(folder) if os.path.isdir(os.path.join(folder,_))]
	setfolderpaths = [os.path.join(folder,_) for _ in setfolders]
	exp = os.path.split(os.path.split(folder)[0])[0]
	expname = [os.path.split(os.path.split(folder)[0])[1]]*len(setfolders)
	d= {'condition': expname,
		'set number':setfolders,
		'set location':setfolderpaths,
		'note':['None']*len(setfolders),
		'ch1':['ch1']*len(setfolders),
		'ch2':['ch2']*len(setfolders)}	
	dd = dict([ (k,pd.Series(v)) for k,v in d.items()])
	df = pd.DataFrame(data=dd)
	df = df[['condition','set number','set location','note','ch1','ch2']]
	df.to_csv(os.path.join(folder , 'segmented image set location.csv'), index=False)

	if not os.path.exists(os.path.join(exp,'image sets for building model.csv')):
		modeling_ledger(exp)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="picellcounter",
    version="1.0.4",
    author="Kyu Sang Han",
    author_email="khan21@jhu.edu",
    description="Raspberry Pi CellCounter",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://wirtzlab.johnshopkins.edu",
    packages=setuptools.find_packages(),
    install_requires=[
        'scipy==1.2.1',
        'pandas==0.24.1', 
        'numpy==1.16.2', 
        'pillow==5.4.1',
        'matplotlib==2.2.4', 
        'scikit-learn==0.20.3', 
        'scikit-image==0.14.2',
        'opencv-python==4.0.0.21',
    ],
    scripts=['bin/cellcounter.py'],
    classifiers=(
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS :: MacOS X",
    ),
)